@@ -9,13 +9,14 @@ use parent 'Mail::Reporter';
 use strict;
 use warnings;
 
+use Log::Report      'mail-box', import => [ qw/__x error info mistake panic trace warning/ ];
+
 use Mail::Box::Message     ();
 use Mail::Box::Locker      ();
 
 use Scalar::Util           qw/weaken/;
 use List::Util             qw/sum first/;
 use Devel::GlobalDestruction 'in_global_destruction';
-use Carp;
 
 #--------------------
 =chapter NAME
@@ -66,6 +67,12 @@ A Mail::Box::Manager creates C<Mail::Box> objects.  But you already
 knew, because you started with the M<Mail::Box-Overview> manual page.
 That page is obligatory reading, sorry!
 
+B<Be aware:>
+This module versions 4.0 and up is not fully compatible with older releases:
+mainly the exception handling has changed.  When you need to upgrade, please
+read F<https://github.com/markov2/perl5-Mail-Box/wiki/>
+B<Version 3 is still maintained> and may see new releases as well.
+
 C<Mail::Box> is the base class for accessing various types of mailboxes
 (folders) in a uniform manner.  The various folder types vary on how
 they store their messages, but when some effort those differences could
@@ -155,7 +162,7 @@ work when access is granted for writing or appending to the folder.
 Be careful: you may create a different folder type than you expect unless you
 explicitly specify M<Mail::Box::Manager::open(type)>.
 
-=option  folder FOLDERNAME
+=option  folder $name
 =default folder C<$ENV{MAIL}>
 Which folder to open (for reading or writing). When used for reading (the
 P<access> option set to C<"r"> or C<"a">) the mailbox should already exist
@@ -376,10 +383,9 @@ added to the body of the message.  With this flag set, the erroneous line
 is added to the previous header field and parsing is continued.
 See M<Mail::Box::Parser::Perl::new(fix_header_errors)>.
 
-=error No folder name specified.
+=error no folder name specified.
 You did not specify the name of a folder to be opened.  Use the
 M<new(folder)> option or set the C<MAIL> environment variable.
-
 =cut
 
 sub new(@)
@@ -388,15 +394,16 @@ sub new(@)
 	if($class eq __PACKAGE__)
 	{	my $package = __PACKAGE__;
 
-		croak <<USAGE;
+		print STDERR <<USAGE;
 You should not instantiate $package directly, but rather one of the
 sub-classes, such as Mail::Box::Mbox.  If you need automatic folder
 type detection then use Mail::Box::Manager.
 USAGE
+		exit 1;
 	}
 
 	weaken $args{manager};   # otherwise, the manager object may live too long
-	my $self = $class->SUPER::new(%args, init_options => \%args) or return;
+	my $self = $class->SUPER::new(%args, init_options => \%args);
 
 	$self->read or return
 		if $self->{MB_access} =~ /r|a/;
@@ -406,11 +413,11 @@ USAGE
 
 sub init($)
 {	my ($self, $args) = @_;
-	defined $self->SUPER::init($args) or return;
+	$self->SUPER::init($args);
 
 	my $class      = ref $self;
 	my $foldername = $args->{folder} || $ENV{MAIL}
-		or $self->log(ERROR => "No folder name specified."), return;
+		or error __x"no folder name specified.";
 
 	$self->{MB_foldername}   = $foldername;
 	$self->{MB_init_options} = $args->{init_options};
@@ -463,7 +470,6 @@ sub init($)
 		timeout  => $args->{lock_timeout},
 		expires  => $args->{lock_wait},
 		file     => ($args->{lockfile} || $args->{lock_file}),
-		$self->logSettings,
 	);
 
 	$self;
@@ -609,6 +615,7 @@ of the folders.... but maybe not, depending on the folder types involved.
   $folder->addMessage($msg);
   $folder->addMessages($msg1, $msg2, ...);
 
+=error you cannot add a message which is already part of a folder to a new one.  Please use moveTo or copyTo.
 =cut
 
 sub addMessage($@)
@@ -616,10 +623,8 @@ sub addMessage($@)
 	my $message = shift or return $self;
 	my %args    = @_;
 
-	confess <<ERROR if $message->can('folder') && defined $message->folder;
-You cannot add a message which is already part of a folder to a new
-one.  Please use moveTo or copyTo.
-ERROR
+	$message->can('folder') && defined $message->folder
+		and error __x"you cannot add a message which is already part of a folder to a new one.  Please use moveTo or copyTo.";
 
 	# Force the message into the right folder-type.
 	my $coerced = $self->coerce($message);
@@ -696,20 +701,15 @@ folder types do support it by creating a hardlink (on UNIX/Linux).
   $imap->copyTo($mh, delete_copied => 1);
   $mh->close; $imap->close;
 
-=error Destination folder $name is not writable.
+=error destination folder $name is not writable.
 The folder where the messages are copied to is not opened with write
 access (see M<new(access)>).  This has no relation with write permission
 to the folder which is controlled by your operating system.
 
-=error Copying failed for one message.
-For some reason, for instance disc full, removed by external process, or
+=error copying failed for one message.
+For some reason, for instance disk full, removed by external process, or
 read-protection, it is impossible to copy one of the messages.  Copying will
 proceed for the other messages.
-
-=error Unable to create subfolder $name of $folder.
-The copy includes the subfolders, but for some reason it was not possible
-to copy one of these.  Copying will proceed for all other sub-folders.
-
 =cut
 
 sub copyTo($@)
@@ -738,15 +738,15 @@ sub _copy_to($@)
 	my ($select, $flatten, $recurse, $delete, $share) = @options;
 
 	$to->writable
-		or $self->log(ERROR => "Destination folder $to is not writable."), return;
+		or error __x"destination folder {name} is not writable.", name => $to;
 
 	# Take messages from this folder.
 	my @select = $self->messages($select);
-	$self->log(PROGRESS => "Copying ".@select." messages from $self to $to.");
+	trace "Copying ".@select." messages from $self to $to.";
 
 	foreach my $msg (@select)
 	{	$msg->copyTo($to, share => $share)
-			or $self->log(ERROR => "Copying failed for one message.");
+			or error __x"copying failed for one message.";
 
 		$msg->label(deleted => 1) if $delete;
 	}
@@ -756,10 +756,9 @@ sub _copy_to($@)
 
 	# Take subfolders
 
-SUBFOLDER:
+  SUBFOLDER:
 	foreach my $subf ($self->listSubFolders(check => 1))
-	{	my $subfolder = $self->openSubFolder($subf, access => 'r')
-			or $self->log(ERROR => "Unable to open subfolder $subf"), next;
+	{	my $subfolder = $self->openSubFolder($subf, access => 'r');
 
 		if($flatten)   # flatten
 		{	unless($subfolder->_copy_to($to, @options))
@@ -768,9 +767,7 @@ SUBFOLDER:
 			}
 		}
 		else           # recurse
-		{	my $subto = $to->openSubFolder($subf, create => 1, access => 'rw')
-				or $self->log(ERROR => "Unable to create subfolder $subf of $to"), next SUBFOLDER;
-
+		{	my $subto = $to->openSubFolder($subf, create => 1, access => 'rw');
 			unless($subfolder->_copy_to($subto, @options))
 			{	$subfolder->close;
 				$subto->close;
@@ -828,7 +825,9 @@ the folder may remove the messages for real.  See M<write(save_deleted)>.
   $f->close
       or die "Couldn't write $f: $!\n";
 
-=warning Changes not written to read-only folder $self.
+=error unknown value to folder->close(write => $when).
+
+=warning changes not written to read-only folder $name.
 You have opened the folder read-only --which is the default set
 by M<new(access)>--, made modifications, and now want to close it.
 Set M<close(force)> if you want to overrule the access mode, or close
@@ -852,11 +851,11 @@ sub close(@)
 	  = $when eq 'MODIFIED' ? $self->isModified
 	  : $when eq 'ALWAYS'   ? 1
 	  : $when eq 'NEVER'    ? 0
-	  :   croak "Unknown value to folder->close(write => $when).";
+	  :    error __x"unknown value to folder->close(write => {when}).", when => $when;
 
 	my $locker = $self->locker;
 	if($write && !$force && !$self->writable)
-	{	$self->log(WARNING => "Changes not written to read-only folder $self; suggestion: \$folder->close(write => 'NEVER')");
+	{	warning __x"changes not written to read-only folder {name}; suggestion: \$folder->close(write => 'NEVER')", name => $self->name;
 		$locker->unlock if $locker;
 		$self->{MB_messages} = [];    # Boom!
 		return 0;
@@ -880,7 +879,7 @@ destination folder before deleting the source folder.  Otherwise you may lose
 data if the system crashes or if there are software problems.
 
 =option  recursive BOOLEAN
-=default recursive 1
+=default recursive true
 
 =example removing an open folder
   my $folder = Mail::Box::Mbox->new(folder => 'InBox', access => 'rw');
@@ -891,7 +890,7 @@ data if the system crashes or if there are software problems.
   my $folder = Mail::Box::Mbox->new(folder => 'INBOX', access => 'd');
   $folder->delete;
 
-=error Folder $name not deleted: not writable.
+=warning folder $name not deleted: not writable.
 The folder must be opened with write access via M<new(access)>, otherwise
 removing it will be refused.  So, you may have write-access according to
 the operating system, but that will not automatically mean that this
@@ -905,7 +904,7 @@ sub delete(@)
 
 	# Extra protection: do not remove read-only folders.
 	unless($self->writable)
-	{	$self->log(ERROR => "Folder $self not deleted: not writable.");
+	{	warning __x"folder {name} not deleted: not writable.", name => $self->name;
 		$self->close(write => 'NEVER');
 		return;
 	}
@@ -933,22 +932,22 @@ be used for appending.  This can be fast, but this can also be very
 slow (depends on the implementation).  All %options passed will also be
 used to open the folder, if needed.
 
-=requires folder FOLDERNAME
-The name of the folder to which the messages are to be appended.  The folder
+=requires folder $name
+The $name of the folder to which the messages are to be appended.  The folder
 implementation will avoid opening the folder when possible, because this is
 resource consuming.
 
-=option  message MESSAGE
+=option  message $message
 =default message undef
 
-=option  messages ARRAY-OF-MESSAGES
+=option  messages $message|\@messages
 =default messages undef
-One reference to a MESSAGE or a reference to an ARRAY of MESSAGEs, which may
+One reference to one $message or an ARRAY of @messages, which may
 be of any type.  The messages will be first coerced into the correct
 message type to fit in the folder, and then will be added to it.
 
 =option  share BOOLEAN
-=default share <false>
+=default share false
 Try to share physical storage of the message.  Only available for a
 limited number of folder types, otherwise no-op.
 
@@ -1002,7 +1001,7 @@ sub modified(;$)
 	return
 		if $self->{MB_modified} = shift;    # force modified flag
 
-	# unmodify all messages
+	# Unmodify all messages as well
 	$_->modified(0) for $self->messages;
 	0;
 }
@@ -1014,7 +1013,6 @@ were added after the folder was read from file.
 
 WARNING: this flag is not related to an external change to the folder
 structure on disk.  Have a look at M<update()> for that.
-
 =cut
 
 sub isModified()
@@ -1025,6 +1023,7 @@ sub isModified()
 	{	return $self->{MB_modified} = 1
 			if $msg->isDeleted || $msg->isModified;
 	}
+$self->{MB_modified} = first { $_->isModified } @{$self->{MB_messages}};
 
 	0;
 }
@@ -1074,13 +1073,13 @@ info around the angles will be removed too.
   my $msg = $folder->messageId('complex-message.id');
   my $msg = $folder->messageId('garbage <complex-message.id> trash');
 
-=warning Message-id '$msgid' does not contain a domain.
+=warning message-id '$msgid' does not contain a domain.
 According to the RFCs, message-ids need to contain a unique random part,
 then an C<@>, and then a domain name.  This is made to avoid the creation
 of two messages with the same id.  The warning emerges when the C<@> is
 missing from the string.
 
-=warning Different messages with id $msgid
+=warning different messages with id $msgid.
 The message id is discovered more than once within the same folder, but the
 content of the message seems to be different.  This should not be possible:
 each message must be unique.
@@ -1094,7 +1093,7 @@ sub messageId($;$)
 	{	$msgid = $1 =~ s/\s//grs;
 
 		index($msgid, '@') >= 0
-			or $self->log(WARNING => "Message-id '$msgid' does not contain a domain.");
+			or warning __x"message-id '{msgid}' does not contain a domain.", msgid => $msgid;
 	}
 
 	@_ or return $self->{MB_msgid}{$msgid};
@@ -1122,7 +1121,7 @@ sub messageId($;$)
 		return $message->label(deleted => 1)
 			if $subj1 eq $subj2 && $to1 eq $to2;
 
-		$self->log(WARNING => "Different messages with id $msgid");
+		warning __x"different messages with id {msgid}.", msgid => $msgid;
 		$msgid = $message->takeMessageId(undef);
 	}
 
@@ -1258,7 +1257,7 @@ to be read.  See their respective manual pages.
 
 =cut
 
-sub messageIds()    { map $_->messageId,  $_[0]->messages }
+sub messageIds()    { map $_->messageId, $_[0]->messages }
 sub allMessageIds() { $_[0]->messageIds }  # compatibility
 sub allMessageIDs() { $_[0]->messageIds }  # compatibility
 
@@ -1388,10 +1387,10 @@ sub scanForMessages($$$$)
 	keys %search;
 }
 
-=method findFirstLabeled $label, [BOOLEAN, [$msgs]]
+=method findFirstLabeled $label, [BOOLEAN, [\@msgs]]
 Find the first message which has this $label with the correct setting. The
 BOOLEAN indicates whether any true value or any false value is to be
-found in the ARRAY of $msgs.  By default, a true value is searched for.
+found in the ARRAY of @msgs.  By default, a true value is searched for.
 When a message does not have the requested label, it is taken as false.
 
 =examples looking for a labeled message
@@ -1422,21 +1421,21 @@ access to that folder.
 
 For MBOX folders, sub-folders are simulated.
 
-=option  folder FOLDERNAME
+=option  folder $name
 =default folder <from calling object>
 The folder whose sub-folders should be listed.
 
-=option  folderdir DIRECTORY
+=option  folderdir $dir
 =default folderdir <from folder>
 
 =option  check BOOLEAN
-=default check <false>
+=default check false
 Should all returned foldernames be checked to be sure that they are of
 the right type?  Each sub-folder may need to be opened to check this,
 with a folder type dependent penalty (in some cases very expensive).
 
 =option  skip_empty BOOL
-=default skip_empty <false>
+=default skip_empty false
 Shall empty folders (folders which currently do not contain any messages)
 be included?  Empty folders are not useful to open, but may be useful
 to save to.
@@ -1551,11 +1550,7 @@ sub read(@)
 		@_
 	) or return;
 
-	if($self->{MB_modified})
-	{	$self->log(INTERNAL => "Modified $self->{MB_modified}");
-		$self->{MB_modified} = 0;  #after reading, no changes found yet.
-	}
-
+	$self->{MB_modified} and panic "Modified $self->{MB_modified}";
 	$self;
 }
 
@@ -1587,19 +1582,13 @@ the folder may remove the messages for real.  See M<close(save_deleted)>.
 =error Folder $name is opened read-only
 You can not write to this folder unless you have opened the folder to
 write or append with M<new(access)>, or the P<force> option is set true.
-
-=error Writing folder $name failed
-For some reason (you probably got more error messages about this problem)
-it is impossible to write the folder, although you should because there
-were changes made.
-
 =cut
 
 sub write(@)
 {	my ($self, %args) = @_;
 
 	$args{force} || $self->writable
-		or $self->log(ERROR => "Folder $self is opened read-only."), return;
+		or error __x"folder {name} is opened read-only.", name => $self->name;
 
 	my (@keep, @destroy);
 	if($args{save_deleted})
@@ -1616,12 +1605,10 @@ sub write(@)
 	}
 
 	@destroy || $self->isModified
-		or $self->log(PROGRESS => "Folder $self not changed, so not updated."), return $self;
+		or trace("Folder $self not changed, so not updated."), return $self;
 
 	$args{messages} = \@keep;
-	$self->writeMessages(\%args)
-		or $self->log(WARNING => "Writing folder $self failed."), return undef;
-
+	$self->writeMessages(\%args);
 	$self->modified(0);
 	$self->{MB_messages} = \@keep;
 	$self;
@@ -1825,7 +1812,7 @@ Parameter $time is a string, which starts with a float, and then one of the
 words 'hour', 'hours', 'day', 'days', 'week', or 'weeks'.  For instance:
 '1 hour' or '4 weeks'.
 
-=error Invalid timespan '$timespan' specified.
+=error invalid timespan '$span'.
 The string does not follow the strict rules of the time span syntax which
 is permitted as parameter.
 =cut
@@ -1833,7 +1820,7 @@ is permitted as parameter.
 sub timespan2seconds($)
 {
 	$_[1] =~ /^\s*(\d+\.?\d*|\.\d+)\s*(hour|day|week)s?\s*$/
-		or $_[0]->log(ERROR => "Invalid timespan '$_[1]' specified."), return undef;
+		or error(__x"invalid timespan '{span}'.", span => $_[1]), return undef;
 
 	  $2 eq 'hour' ? $1 * 3600
 	: $2 eq 'day'  ? $1 * 86400
